@@ -1,44 +1,75 @@
 package OpenTracing::Role::Span;
 
-our $VERSION = 'v0.86.1';
+=head1 NAME
 
+OpenTracing::Role::Span - Role for OpenTracing implementations.
 
+=head1 SYNOPSIS
+
+    package OpenTracing::Implementation::MyBackendService::Span;
+    
+    use Moo;
+    
+    ...
+    
+    with 'OpenTracing::Role::Span'
+    
+    1;
+
+=cut
 
 use Moo::Role;
 use MooX::HandlesVia;
-use MooX::ProtectedAttributes;
-use MooX::Should;
 
 use Carp;
-use OpenTracing::Types qw/:types :is/;
-use Time::HiRes qw/time/;
-use Types::Standard qw/CodeRef HashRef Maybe Num Object Str Value/;
-use Types::Common::Numeric qw/PositiveOrZeroNum/;
+use Types::Standard qw/HashRef Num Object Str Value/;
+use Types::Interface qw/ObjectDoesInterface/;
+
+
+
+=head1 DESCRIPTION
+
+This is a Role for OpenTracing implenetations that are compliant with the
+L<OpenTracing::Interface>.
+
+With the exception of calls to C<get_context()> (which are always allowed),
+C<finish()> must be the last call made to any span instance, and to do otherwise
+leads to undefined behavior (but not returning an exception).
+
+=cut
+
+
 
 has operation_name => (
     is              => 'rwp',
-    should          => Str,
+    isa             => Str,
     required        => 1,
 #   writer          => 'overwrite_operation_name',
-    reader          => 'get_operation_name', # it's not in the Interface
+#   reader          => 'get_operation_name', # not sure, it's not in the Interface
 );
 
+
+
 has start_time => (
     is              => 'ro',
-    should          => PositiveOrZeroNum,
+    isa             => Num,
     default         => sub { epoch_floatingpoint() }
 );
 
+
+
 has finish_time => (
     is              => 'rwp',
-    should          => PositiveOrZeroNum,
+    isa             => Num,
     predicate       => 'has_finished',
     init_arg        => undef,
 );
 
+
+
 has tags => (
     is              => 'rwp',
-    should          => HashRef[Value],
+    isa             => HashRef[Value],
     handles_via     => 'Hash',
     handles         => {
         get_tags => 'all',
@@ -46,17 +77,18 @@ has tags => (
     default         => sub{ {} },
 );
 
+
+
 has context => (
     is              => 'ro',
-    should          => SpanContext,
+    isa             => ObjectDoesInterface['OpenTracing::Role::SpanContext'],
     reader          => 'get_context',
 #   writer          => '_set_context',
     required        => 1, # either from Span->get_context or SpanContext self
-    handles         => {
-        get_span_id     => 'span_id',
-    },
 );
 
+
+
 sub overwrite_operation_name {
     my $self = shift;
     
@@ -70,23 +102,24 @@ sub overwrite_operation_name {
     return $self
 }
 
+
+
 sub finish {
     my $self = shift;
     
-    carp "Span has already been finished" and return $self
+    croak "Tag has already been finished"
         if $self->has_finished;
     
     my $epoch_timestamp = shift // epoch_floatingpoint();
     
     $self->_set_finish_time( $epoch_timestamp );
     
-    $self->on_finish->( $self )
-        if $self->has_on_finish;
-    
     return $self
 }
 
-sub add_tag {
+
+
+sub set_tag {
     my $self = shift;
     
     croak "Can't set a tag on an already finished span"
@@ -95,24 +128,12 @@ sub add_tag {
     my $key = shift;
     my $value = shift;
     
-    $self->add_tags( $key => $value );
+    $self->{ tags }->{ $key } = $value;
     
     return $self
 }
 
-sub add_tags {
-    my $self = shift;
-    
-    croak "Can't set a tag on an already finished span"
-        if $self->has_finished;
-    
-    my %tags = @_;
-    $self->_set_tags(
-        { $self->get_tags, %tags }
-    );
-    
-    return $self
-}
+
 
 sub log_data {
     my $self = shift;
@@ -127,7 +148,9 @@ sub log_data {
     return $self
 }
 
-sub add_baggage_item {
+
+
+sub set_baggage_item {
     my $self = shift;
     
     croak "Can't set baggage-items on an already finished span"
@@ -136,12 +159,15 @@ sub add_baggage_item {
     my $key = shift;
     my $value = shift;
     
-    $self->add_baggage_items( $key => $value );
+    my $new_context = $self->get_context()->with_baggage_item( $key, $value );
+    $self->_set_context( $new_context );
     
     return $self
 }
 
-sub add_baggage_items {
+
+
+sub set_baggage_items {
     my $self = shift;
     
     croak "Can't set baggage-items on an already finished span"
@@ -155,6 +181,8 @@ sub add_baggage_items {
     return $self
 }
 
+
+
 sub get_baggage_item {
     my $self = shift;
     my $key = shift;
@@ -162,11 +190,7 @@ sub get_baggage_item {
     return $self->get_context()->get_baggage_item( $key )
 }
 
-sub get_baggage_items {
-    my $self = shift;
-    
-    return $self->get_context()->get_baggage_items
-}
+
 
 sub duration { 
     my $self = shift;
@@ -175,59 +199,21 @@ sub duration {
         or croak
             "Span has not been started: ['"
             .
-            ( $self->get_operation_name || "'undef'" )
+            $self->operation_name
             .
             "'] ... how did you do that ?";
     my $finish_time = $self->{ finish_time }
         or croak
             "Span has not been finished: ['"
             .
-            ( $self->get_operation_name || "'undef'" )
+            $self->operation_name
             .
             "'] ... yet!";
     
     return $finish_time - $start_time
 }
 
-protected_has child_of => (
-    is => 'ro',
-    should => Span | SpanContext,
-    required => 0,
-);
-#
-# this is just non of your business, and will get depricated as soon as there is
-# references
 
-sub _get_child_of { $_[0]->child_of }
-#
-# so this can be swapped for something more clever once using references
-
-sub get_parent_span_id {
-    my $self = shift;
-    
-    my $child_of = $self->_get_child_of;
-    
-    return unless defined $child_of;
-    
-    return $child_of->span_id
-        if is_SpanContext($child_of);
-    
-    return $child_of->get_context->span_id
-        if is_Span($child_of);
-    
-    croak "No 'parent span_id' for 'child_of' attribute [$child_of]"
-    #
-    # execution should never end up here
-    
-}
-#
-# This may not be the right way to implement it, for the `child_of` attribute
-# may not be such a good idea, maybe it should use references, but not sure how
-# those are used
-
-
-
-sub is_root_span { ! shift->get_parent_span_id() }
 
 # _set_context
 #
@@ -246,28 +232,16 @@ sub _set_context {
     return $self
 }
 
-has on_finish => (
-    is              => 'ro',
-    should          => Maybe[CodeRef],
-    predicate       => 1,
-);
 
-sub DEMOLISH {
-    my $self = shift;
-    my $in_global_destruction = shift;
-    
-    return if $self->has_finished;
-    
-#   carp "Span not programmatically finished before being demolished";
-    
-    $self->finish( )
-        unless $in_global_destruction;
-    
-    return
-}
+
+=head2 epoch_floatingpoint
+
+Well, returns the time since 'epoch' with fractional seconds, as floating-point.
+
+=cut
 
 sub epoch_floatingpoint {
-    return time()
+    return scalar gettimeofday()
 }
 #
 # well, this is a bit off a silly idea:
@@ -281,8 +255,11 @@ sub epoch_floatingpoint {
 BEGIN {
 #   use Role::Tiny::With;
     with 'OpenTracing::Interface::Span'
+        if $ENV{OPENTRACING_INTERFACE};
 }
 
 
 
 1;
+
+
